
local _ENV = {
	tonumber = tonumber,
	type = type,
	error = error,
	select = select,
	string = {
		format = string.format,
		find = string.find,
	},
	debug = {
		getinfo = debug.getinfo,
	},
}

local function bad_arg(field, arg_count, mean, expected_type, got_type, level_plus)
	level_plus = tonumber(level_plus) or 0
	local dbinfo = debug.getinfo(2 + level_plus)
	local ftype = (type(dbinfo.namewhat)=="string" and dbinfo.namewhat) or ""
	local fname = (type(dbinfo.name)=="string" and dbinfo.name) or (type(field)=="string" and field) or "?"
	if got_type~="" then
		got_type = string.format(", got %s", got_type)
	end
	return error(string.format(":%d: bad argument #%d%s to %s '%s' (%s expected%s)", debug.getinfo(3 + level_plus).currentline, arg_count, mean, ftype, fname, expected_type, got_type), 3 + level_plus)
end

local function check_value(arg_count, expected_type, ...)
	if type(arg_count)~="number" then
		bad_arg("check_value", 1, "(arg_count)", "number", type(arg_count))
	end
	if type(expected_type)~="string" then
		bad_arg("check_value", 2, "(expected_type)", "string", type(expected_type))
	end
	local argc = select("#", ...)
	if argc<arg_count then
		return bad_arg("", arg_count, "", expected_type, "no value")
	else
		local the_value = select(arg_count, ...)
		if string.find(expected_type, "number") and tonumber(the_value) then
			return tonumber(the_value)
		elseif not string.find(expected_type, type(the_value)) then
			return bad_arg("", arg_count, "", expected_type, type(the_value))
		else
			return the_value
		end
	end
end

local function opt_value(arg_count, expected_type, default_val, ...)
	if type(arg_count)~="number" then
		bad_arg("opt_value", 1, "(arg_count)", "number", type(arg_count))
	end
	if type(expected_type)~="string" then
		bad_arg("opt_value", 2, "(expected_type)", "string", type(expected_type))
	end
	if (not string.find(expected_type, type(default_val))) and type(default_val)~="nil" then
		bad_arg("opt_value", 3, "(default_val)", expected_type, type(default_val))
	end
	local argc = select("#", ...)
	if argc<arg_count then
		return default_val
	else
		local the_value = select(arg_count, ...)
		if string.find(expected_type, "number") and tonumber(the_value) then
			return tonumber(the_value)
		elseif not string.find(expected_type, type(the_value)) then
			return bad_arg("", arg_count, "", "opt."..expected_type, type(the_value))
		else
			return the_value
		end
	end
end

return {
	check_value = check_value,
	opt_value = opt_value,
}

