local _M = {}

local Java = _G.Java

local argth = require("scripts.argth")
local check_value = argth.check_value
local opt_value = argth.opt_value

local cbutils = require("scripts.colorpicker.utils.callbacks")
local registerCallback = cbutils.registerCallback
local customformats = require("scripts.colorpicker.init.customformats")
local getMouseTrackInfoBottomRule = customformats.getMouseTrackInfoBottomRule
local getMouseTrackInfoZoomRule = customformats.getMouseTrackInfoZoomRule
local getMultiPosFormatRule = customformats.getMultiPosFormatRule
local getSinglePosFormatRule = customformats.getSinglePosFormatRule
local getMakeScriptRules = customformats.getMakeScriptRules
local getSettingsRule = customformats.getSettingsRule
local getRightClickRules = customformats.getRightClickRules
local getSelectedFormatName = customformats.getSelectedFormatName
local getCurrentConfig = customformats.getCurrentConfig
local saveConfig = customformats.saveConfig

local Java_dispatchEvent = Java.dispatchEvent
local Java_newThread = Java.newThread
local Java_new = Java.new
local Java_class = Java.class
local Java_imple = Java.imple
local Java_newRunnable = Java.newRunnable
local Java_newActionListener = Java.newActionListener
local Java_newAction = Java.newAction


local package = _G.package

local _1f = _1func
local _P = convPosColor
local getPosList = getPosList

package["1func"] = package["1func"] or {}

function checkAndCallFunc(f)
    if type(f)=="function" then
        pcall(f)
    end
end

registerCallback("onKeyPressed", function(e)
    local keymap = require("scripts.colorpicker.keymap")
    local keycode = e:getKeyCode()
    if e:isAltDown() then
        if e:isShiftDown() then
            if e:isControlDown() then
                checkAndCallFunc(keymap.CTRL_ALT_SHIFT[keycode])
            else
                checkAndCallFunc(keymap.ALT_SHIFT[keycode])
            end
        else
            if e:isControlDown() then
                checkAndCallFunc(keymap.CTRL_ALT[keycode])
            else
                checkAndCallFunc(keymap.ALT[keycode])
            end
        end
    else
        if e:isShiftDown() then
            if e:isControlDown() then
                checkAndCallFunc(keymap.CTRL_SHIFT[keycode])
            else
                checkAndCallFunc(keymap.SHIFT[keycode])
            end
        else
            if e:isControlDown() then
                checkAndCallFunc(keymap.CTRL[keycode])
            else
                checkAndCallFunc(keymap[1][keycode])
            end
        end
    end
end)

registerCallback("onKeyReleased", function(e)

end)

registerCallback("onMouseTrack", function(e)
    local p = _P(_1f:getCurrentPos())
    _1f:setImageStatusTips(getMouseTrackInfoBottomRule()(p))
    _1f:setZoomInfoText(getMouseTrackInfoZoomRule()(p))
end)

local clearLastPick = _G.clearLastPick
local pickToNextColorRegister = _G.pickToNextColorRegister
registerCallback("onMouseClicked", function(e)
    if Java_class("javax.swing.SwingUtilities"):isLeftMouseButton(e) then
        pickToNextColorRegister()
    elseif Java_class("javax.swing.SwingUtilities"):isRightMouseButton(e) then
        clearLastPick()
    end
end)

local isMac = false
local osname = Java_class("java.lang.System"):getProperties():getProperty("os.name")
if (osname:find("Mac") or osname:find("mac")) then
    isMac = true
end

registerCallback("onMouseWheelMoved", function(e)
    local t = 1
    if (isMac) then
        t = -1
    end
    local p = _1f:getCurrentPos()
    if (e:getModifiers() == 1) then
        _1f:moveTo(p:getX() + (e:getWheelRotation() * t), p:getY())
    else
        _1f:moveTo(p:getX(), p:getY() + (e:getWheelRotation() * t))
    end
end)

local clearColorRegisterByIndex = _G.clearColorRegisterByIndex
local clearAllColorRegisters = _G.clearAllColorRegisters

registerCallback("onClearButtonClick", function(idx)
    if idx ~= 10 then
        clearColorRegisterByIndex(idx)
    else
        clearAllColorRegisters()
    end
end)

local function tableCellTextCopy(event, row, col)
    local t = event:getSource()
    local str = t:getValueAt(row,t:getColumnCount() - 1)
--    Java_dispatchEvent(function()
--        _1f:getTheEditor():insert(str, _1f:getTheEditor():getCaretPosition())
--    end)
    _1f:writePasteboard(str)
--    _1f:print(string.format("line %d has been copyed.\n", row+1))
end

local tableSelectionBackground
local WHITE_COLOR = Java_new("java.awt.Color", 1, 1, 1)

registerCallback("onResultsTableDoubleClickCallback", function(event, row, col)
    tableCellTextCopy(event, row, col)
    local tab = event:getSource()
    tableSelectionBackground = tableSelectionBackground or tab:getSelectionBackground()
    Java_newThread(function()
        Java_dispatchEvent(function()
            tab:setSelectionBackground(WHITE_COLOR)
        end)
        Java.class("java.lang.Thread"):sleep(150)
        Java_dispatchEvent(function()
            tab:setSelectionBackground(tableSelectionBackground)
        end)
    end):start()
end)

registerCallback("onResultsTableRightClickCallback", function(event, row, col)
    local t = event:getSource()
    local popup = Java_new("javax.swing.JPopupMenu")

    local rules = getMakeScriptRules()

    local menutab

    if type(rules[row + 1])=="table" and type(rules[row + 1].menu)=="table" then
        menutab = rules[row + 1].menu
    elseif type(rules[row + 1])=="table" and type(rules[row + 1].menu)=="function" then
        local poslist = getPosList()
        local set = getCurrentConfig()
        local ctx = t:getValueAt(row,t:getColumnCount() - 1)
        local noerr
        noerr, menutab = pcall(rules[row + 1].menu, ctx, row + 1, poslist, set)
        if (not noerr) or type(menutab)~="table" then
            return
        end
    else
        return
    end

    for _, v in ipairs(menutab) do
        if type(v)=="table" and type(v.action)=="function" then
            local item
            local ctx = t:getValueAt(row,t:getColumnCount() - 1)
            item = Java_new("javax.swing.JMenuItem", v.name or "(unnamed)")
            item:addActionListener(Java_newActionListener(function()
                local poslist = getPosList()
                local set = getCurrentConfig()
                v.action(ctx, row + 1, poslist, set)
            end))
            popup:add(item)
        end
    end

    popup:show(t, event:getX(), event:getY())
end)

registerCallback("onMakeScriptsButtonClick", _G.makeScripts)

local getTextWithRegister = _G.getTextWithRegister
registerCallback("onTextFieldDoubleClick", function(idx)
    _1f:writePasteboard(getTextWithRegister(idx))
    print(string.format("内容 '%s' 已经复制到剪贴板\n", getTextWithRegister(idx)))
end)

registerCallback("onColorLabelClick", function(idx)
    local pc = _1f:readColorRegister(idx)
    _1f:writePasteboard(string.format("0x%06x", pc:getRGB()))
    print(string.format("颜色 '0x%06x' 已经复制到剪贴板\n", pc:getRGB()))
end)

registerCallback("onColorLabelDoubleClick", function(idx)
    local pc = _1f:readColorRegister(idx)
    _1f:writePasteboard(string.format("%d, %d", pc:getX(), pc:getY()))
    print(string.format("坐标 '%d, %d' 已经复制到剪贴板\n", pc:getX(), pc:getY()))
end)

registerCallback("onConfButtonClick", function()
    local settingsRule = getSettingsRule()
    local setin = {}
    local set = getCurrentConfig()
    for _, v in ipairs(settingsRule.args) do
        setin[#setin + 1] = {v[1], set[v[1]], v[3], v[4]}
    end
    local setout = dialogConfig(settingsRule.title, setin)
    saveConfig(setout)
end)

local ASXCButtonSwitcher = {
    A = function()
        local p = _P(_1f:getPosA())
        _1f:writePasteboard(p.x .. ", " .. p.y)
    end,
    S = function()
        local p = _P(_1f:getPosS())
        _1f:writePasteboard(p.x .. ", " .. p.y)
    end,
    X = function()
        local p = _P(_1f:getPosX())
        _1f:writePasteboard(p.x .. ", " .. p.y)
    end,
    C = function()
        local p = _P(_1f:getPosC())
        _1f:writePasteboard(p.x .. ", " .. p.y)
    end,
}

registerCallback("onASXCButtonClick", function(name)
--    _1f:print(name .. " button has been clicked.\n");
    if type(ASXCButtonSwitcher[name]) == "function" then
        local noerr, errmsg = pcall(ASXCButtonSwitcher[name])
        if not noerr then
            error("onASXCButtonClick('"..tostring(name).."')\n"..errmsg)
        end
    end
end)


