local _M = {}

local Java = _G.Java

local argth = require("scripts.argth")
local check_value = argth.check_value
local opt_value = argth.opt_value

local configOptString = configOptString
local configSetString = configSetString
local configOptBoolean = configOptBoolean
local configSetBoolean = configSetBoolean
local configOptInteger = configOptInteger
local configSetInteger = configSetInteger
local configOptDouble = configOptDouble
local configSetDouble = configSetDouble


local _1f = _1func

local cfutils = require("scripts.colorpicker.utils.customformats")
local getSelectedFormat = cfutils.getSelectedFormat
local getSelectedFormatName = cfutils.getSelectedFormatName

for _, mn in ipairs(cfutils.getModuleNames()) do
    local noerr, m = pcall(require, mn)
    if noerr then
        if not cfutils.registerCustomFormat(m) then
            _1f:print("module '"..mn.."' load failed : invalid module name.")
        end
    else
        _1f:print("module '"..mn.."' load failed : "..m)
    end
end

local function getMouseTrackInfoBottomRule()
    local fmt = getSelectedFormat()
    fmt.mouseTrackInfoBottomRule = fmt.mouseTrackInfoBottomRule or function(p, a, s, x, c, set)
        return string.format("XY: (%4d, %4d)      Color: 0x%06x      RGB: (%3d, %3d, %3d)", p.x, p.y, p.c, p.r, p.g, p.b)
    end
    return fmt.mouseTrackInfoBottomRule
end

local function getMouseTrackInfoZoomRule()
    local fmt = getSelectedFormat()
    fmt.mouseTrackInfoZoomRule = fmt.mouseTrackInfoZoomRule or function(p, a, s, x, c, set)
        return string.format("(%4d,%4d):0x%06x:(%3d,%3d,%3d)", p.x, p.y, p.c, p.r, p.g, p.b)
    end
    return fmt.mouseTrackInfoZoomRule
end

local function getSinglePosFormatRule()
    local fmt = getSelectedFormat()
    fmt.singlePosFormatRule = fmt.singlePosFormatRule or function(p, a, s, x, c, set)
        return string.format(" %4d, %4d, 0x%06x ", p.x, p.y, p.c)
    end
    return fmt.singlePosFormatRule
end

local function getMultiPosFormatRule()
    local fmt = getSelectedFormat()
    fmt.multiPosFormatRule = fmt.multiPosFormatRule or function(p, a, s, x, c, set)
        string.format(" %4d, %4d, 0x%06x \n", p.x, p.y, p.c)
    end
    return fmt.multiPosFormatRule
end

local function getMakeScriptRules()
    local fmt = getSelectedFormat()
    fmt.makeScriptRules = fmt.makeScriptRules or {}
    return fmt.makeScriptRules
end

local function getSettingsRule()
    local fmt = getSelectedFormat()
    fmt.settingsRule = fmt.settingsRule or {}
    return fmt.settingsRule
end

local function getTestRules()
    local fmt = getSelectedFormat()
    fmt.testRules = fmt.testRules or {}
    return fmt.testRules
end

local function getRightClickRules()
    local fmt = getSelectedFormat()
    fmt.rightClickRules = fmt.rightClickRules or {}
    return fmt.rightClickRules
end

local function getCurrentConfig()
    local settingsRule = getSettingsRule()
    local set = {}
    for _, v in ipairs(settingsRule.args) do
        if type(v[2])=="boolean" then
            set[v[1]] = configOptBoolean("CF_"..getSelectedFormatName(), v[1], v[2])
        elseif type(v[2])=="number" then
            set[v[1]] = tonumber(configOptString("CF_"..getSelectedFormatName(), v[1], v[2])) or v[2]
        else
            set[v[1]] = configOptString("CF_"..getSelectedFormatName(), v[1], v[2])
        end
    end
    return set
end

local function saveConfig(setout)
    for k, v in pairs(setout) do
        if type(v)=="boolean" then
            configSetBoolean("CF_"..getSelectedFormatName(), k, v)
        else
            configSetString("CF_"..getSelectedFormatName(), k, tostring(v))
        end
    end
end


_M.getMouseTrackInfoBottomRule = getMouseTrackInfoBottomRule
_M.getMouseTrackInfoZoomRule = getMouseTrackInfoZoomRule
_M.getSinglePosFormatRule = getSinglePosFormatRule
_M.getMultiPosFormatRule = getMultiPosFormatRule
_M.getMakeScriptRules = getMakeScriptRules
_M.getSettingsRule = getSettingsRule
_M.getTestRules = getTestRules
_M.getRightClickRules = getRightClickRules
_M.getCurrentConfig = getCurrentConfig
_M.saveConfig = saveConfig

_M.getSelectedFormatName = getSelectedFormatName

return _M
