function initRequire(_G)
    local old_require = _G.require
    local pcall = _G.pcall
    local string_gsub = _G.string.gsub
    local package_preload = _G.package.preload
    local load = _G.load
    local _1f = _1func
    return function(mn)
        assert(type(mn)=="string", "bad argument #1 to 'require', string expected")
        local noerr, mormsg = pcall(old_require, mn)
        if noerr then
            return mormsg
        else
            local fn = string_gsub(mn, "[.]", "/")
            fn = "/"..fn..".lua"
            package_preload[mn] = load(_1func:getResourceData(fn),fn)
            local noerr, m = pcall(old_require, mn)
            if noerr then
                return m
            else
                return old_require(mn)
            end
        end
    end
end
_G.require = initRequire(_G)

local luajava_newInstance = luajava.newInstance
local luajava_new = luajava.new

local Java = {
    imple = luajava.createProxy,
    class = luajava.bindClass,
}
_G.Java = Java

local _1f = _1func

local argth = require("scripts.argth")
local check_value = argth.check_value
local opt_value = argth.opt_value

local jRunnableN = "java.lang.Runnable"
local jThreadN = "java.lang.Thread"
local jEventQueueN = "java.awt.EventQueue"
local jAbstractActionN = "javax.swing.AbstractAction"
local jActionListenerN = "java.awt.event.ActionListener"

local jEventQueue = Java.class(jEventQueueN)
local jThread = Java.class(jThreadN)

Java.new = function(...)
    local class = check_value(1, "string.userdata", ...)
    if type(class)=="string" then
        return luajava_newInstance(class, select(2, ...))
    else
        return luajava_new(class, select(2, ...))
    end
end

Java.newRunnable = function(...)
    local func = check_value(1, "function", ...)
    return Java.imple(jRunnableN, {
        run = function()
            func()
        end,
    })
end

Java.newAction = function(...)
    local func = check_value(1, "function", ...)
    return Java.imple(jAbstractActionN, {
        actionPerformed = function(event)
            func(event)
        end,
    })
end

Java.newActionListener = function(...)
    local func = check_value(1, "function", ...)
    return Java.imple(jActionListenerN, {
        actionPerformed = function(event)
            func(event)
        end,
    })
end

function Java.dispatchEvent(...)
    local func = check_value(1, "function", ...)
    jEventQueue:invokeLater(Java.newRunnable(func))
end

function Java.newThread(...)
    local func = check_value(1, "function", ...)
    return Java.new(jThread, Java.newRunnable(func))
end

local package = _G.package
package["1func"] = package["1func"] or {}

function _G.convPosColor(pc)
    local ret = {
        pc:getX(),
        pc:getY(),
        pc:getRGB(),
    }
    ret.x = ret[1]
    ret.y = ret[2]
    ret.c = ret[3]
    ret.r = pc:getR()
    ret.g = pc:getG()
    ret.b = pc:getB()
    return ret
end
local _P = _G.convPosColor

local old_getImageSize = getImageSize
local table_unpack = table.unpack
function _G.getImageSize()
    return table_unpack(old_getImageSize())
end

function _G.copyText(s)
    _1f:writePasteboard(s)
end

function _G.clipText(s)
    _1f:readPasteboard(s)
end

function _G.getCurrentXY()
    local p = _1f:getCurrentPos()
    return p:getX(), p:getY(), _1f:getColor(p:getX(), p:getY()):getRGB()
end

function _G.getCurrentPos()
    local p = _1f:getCurrentPos()
    local c = _1f:getColor(p:getX(), p:getY());
    return {
        p:getX(),
        p:getY(),
        c:getRGB(),
        x = p:getX(),
        y = p:getY(),
        c = c:getRGB(),
        r = c:getR(),
        g = c:getG(),
        b = c:getB(),
    }
end
local getCurrentPos = _G.getCurrentPos

function _G.getColor(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    return _1f:getColor(x, y):getRGB()
end

function _G.getColorRGB(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    local c = _1f:getColor(x, y)
    return c:getR(), c:getG(), c:getB()
end

function _G.getPixel(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    return _1f:getPixel(x, y)
end

function _G.setRegA(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    _1f:setPosAXY(x, y)
end

function _G.setRegS(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    _1f:setPosSXY(x, y)
end

function _G.setRegX(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    _1f:setPosXXY(x, y)
end

function _G.setRegC(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    _1f:setPosCXY(x, y)
end

function _G.getRegA()
    local p = _1f:getPosA()
    return p:getX(), p:getY()
end

function _G.getRegS()
    local p = _1f:getPosS()
    return p:getX(), p:getY()
end

function _G.getRegX()
    local p = _1f:getPosX()
    return p:getX(), p:getY()
end

function _G.getRegC()
    local p = _1f:getPosC()
    return p:getX(), p:getY()
end

function _G.getPosA()
    return _P(_1f:getPosA())
end

function _G.getPosS()
    return _P(_1f:getPosS())
end

function _G.getPosX()
    return _P(_1f:getPosX())
end

function _G.getPosC()
    return _P(_1f:getPosC())
end

function _G.getPosASXC()
    local a = {getRegA()}
    local s = {getRegS()}
    local x = {getRegX()}
    local c = {getRegC()}
    a.x = a[1]
    a.y = a[2]
    s.x = s[1]
    s.y = s[2]
    x.x = x[1]
    x.y = x[2]
    c.x = c[1]
    c.y = c[2]
    return a, s, x, c
end
local getPosASXC = _G.getPosASXC

function _G.getPosList()
    local pl = {}
    local count = 1
    for i = 1, 9 do
        local pc = _1f:readColorRegister(i)
        if pc:getX() ~= -1 and pc:getY() ~= -1 then
            pl[count] = _P(pc)
            pl[count].idx = i
            if i == 0 then
                pl[count].num = 10
            else
                pl[count].num = i
            end
            count = count + 1
        end
    end
    do
        local i = 0
        local pc = _1f:readColorRegister(i)
        if pc:getX() ~= -1 and pc:getY() ~= -1 then
            pl[count] = _P(pc)
            pl[count].idx = i
            if i == 0 then
                pl[count].num = 10
            else
                pl[count].num = i
            end
            count = count + 1
        end
    end
    pl.a = _P(_1f:getPosA())
    pl.s = _P(_1f:getPosS())
    pl.x = _P(_1f:getPosX())
    pl.c = _P(_1f:getPosC())
    if pl.s.x < pl.a.x then
        pl.s.x, pl.a.x = pl.a.x, pl.s.x
    end
    if pl.s.y < pl.a.y then
        pl.s.y, pl.a.y = pl.a.y, pl.s.y
    end
    return pl
end
local getPosList = _G.getPosList

function _G.setTextToRegister(...)
    local idx = check_value(1, "number", ...)
    assert(idx>=0 and idx<=9, "index range 0~9")
    local text = check_value(2, "string", ...)
    _1f:getTextFieldWithIndex(idx):setText(text)
end

function _G.getTextWithRegister(...)
    local idx = check_value(1, "number", ...)
    assert(idx>=0 and idx<=9, "index range 0~9")
    return _1f:getTextFieldWithIndex(idx):getText()
end

function _G.setColorToRegister(...)
    local idx = check_value(1, "number", ...)
    assert(idx>=0 and idx<=9, "index range 0~9")
    local color = check_value(2, "number", ...)
    _1f:setColorLabelColor(idx, color)
end

function _G.writeColorRegister(...)
    local idx = check_value(1, "number", ...)
    assert(idx>=0 and idx<=9, "index range 0~9")
    local x = check_value(2, "number", ...)
    local y = check_value(3, "number", ...)
    local color = check_value(4, "number", ...)
    _1f:writeColorRegister(idx, x, y, color)
end

local customformats = require("scripts.colorpicker.init.customformats")
local getMultiPosFormatRule = customformats.getMultiPosFormatRule
local getCurrentConfig = customformats.getCurrentConfig
local getMakeScriptRules = customformats.getMakeScriptRules

--local function setColorRegister(...)
--    local idx = check_value(1, "number", ...)
--    assert(idx>=0 and idx<=9, "index range 0~9")
--    local x = check_value(2, "number", ...)
--    local y = check_value(3, "number", ...)
--    local color = check_value(4, "number", ...)
--    local p = getCurrentPos()
--    p.c = color
--    local a, s, x, c = getPosASXC()
--    _1f:getTextFieldWithIndex(idx):setText(getMultiPosFormatRule()(p, a, s, x, c, getCurrentConfig()):gsub("^%s*(.-)%s*$", "%1"));
--    _1f:setColorLabelColor(idx, p.c)
--    _1f:writeColorRegister(idx, x, y, p.c)
--end

function _G.reloadPasteboard()
    local txt = ""
    local a, s, x, c = getPosASXC()
    for i = 1, 9 do
        local pc = _1f:readColorRegister(i)
        if pc:getX() ~= -1 and pc:getY() ~= -1 then
            txt = txt..getMultiPosFormatRule()(_P(pc), a, s, x, c, getCurrentConfig())
        end
    end
    do
        local pc = _1f:readColorRegister(0)
        if pc:getX() ~= -1 and pc:getY() ~= -1 then
            txt = txt..getMultiPosFormatRule()(_P(pc), a, s, x, c, getCurrentConfig())
        end
    end
    _1f:writePasteboard(txt)
end
local reloadPasteboard = _G.reloadPasteboard

local lastPickIdx

function _G.pickToColorRegister(...)
    local idx = check_value(1, "number", ...)
    assert(idx>=0 and idx<=9, "index range 0~9")
    local p = getCurrentPos()
    local a, s, x, c = getPosASXC()
    _1f:getTextFieldWithIndex(idx):setText(getMultiPosFormatRule()(p, a, s, x, c, getCurrentConfig()):gsub("^%s*(.-)%s*$", "%1"));
    _1f:setColorLabelColor(idx, p.c)
    _1f:writeColorRegister(idx, p.x, p.y, p.c)
    lastPickIdx = idx
    reloadPasteboard()
end
local pickToColorRegister = _G.pickToColorRegister

function _G.repickColor()
    local pl = getPosList()
    local a, s, x, c = getPosASXC()
    for idx, p in ipairs(pl) do
        local pc = _1f:getColor(p.x, p.y)
        p.c = pc:getRGB()
        p.r = pc:getR()
        p.g = pc:getG()
        p.b = pc:getB()
        p[3] = p.c
        _1f:getTextFieldWithIndex(idx):setText(getMultiPosFormatRule()(p, a, s, x, c, getCurrentConfig()):gsub("^%s*(.-)%s*$", "%1"));
        _1f:setColorLabelColor(idx, p.c)
        _1f:writeColorRegister(idx, p.x, p.y, p.c)
    end
    reloadPasteboard()
end

function _G.pickToNextColorRegister()
    for i = 1, 9 do
        local pc = _1f:readColorRegister(i)
        if pc:getX() == -1 and pc:getY() == -1 then
            pickToColorRegister(i)
            lastPickIdx = i
            return
        end
    end
    do
        local pc = _1f:readColorRegister(0)
        if pc:getX() == -1 and pc:getY() == -1 then
            pickToColorRegister(0)
            lastPickIdx = 0
            return
        end
    end
end

function _G.clearColorRegisterByIndex(idx)
    _1f:getTextFieldWithIndex(idx):setText("")
    _1f:clearColorLabelColor(idx)
    _1f:writeColorRegister(idx, -1, -1, -1)
    if (lastPickIdx == idx) then
        lastPickIdx = nil
    end
    reloadPasteboard()
end
local clearColorRegisterByIndex = _G.clearColorRegisterByIndex

function _G.clearLastPick()
    if lastPickIdx then
        clearColorRegisterByIndex(lastPickIdx)
    end
end

function _G.clearAllColorRegisters()
    for i = 0, 9 do
        _1f:getTextFieldWithIndex(i):setText("")
        _1f:clearColorLabelColor(i)
        _1f:writeColorRegister(i, -1, -1, -1)
    end
    lastPickIdx = nil
    reloadPasteboard()
end

local createTableDataModel = createTableDataModel
function _G.makeScripts()
    local poslist = getPosList()
    local set = getCurrentConfig()
    local data = {}
    local header = {}
    for i, v in ipairs(getMakeScriptRules()) do
        local noerr, s
        if type(v)=="table" and type(v.maker)=="function" then
            noerr, s = pcall(v.maker, poslist, set)
        else
            noerr, s = pcall(v, poslist, set)
        end
        if noerr then
            if type(s) == "string" then
                if i==1 then
                    _1f:writePasteboard(s)
                end
                data[#data + 1] = {s }
                header[1] = ""
            elseif type(s) == "table" then
                data[#data + 1] = s
                for i = 1, #s do
                    header[i] = ""
                end
            else
                _1f:print("'makeScriptRules' returns must be a string.")
            end
        else
            _1f:print(s)
        end
    end
    _1f:getResultsTableCtrl():setRowHeight(36)
    _1f:getResultsTableCtrl():setModel(createTableDataModel(data, header))
end

function _G.viewRotateLeft()
    _1f:imageTurnLeft()
end
_G.imageTurnLeft = _G.viewRotateLeft

function _G.viewRotateRight()
    _1f:imageTurnRight()
end
_G.imageTurnRight = _G.viewRotateRight

function _G.moveMouseToXY(...)
    local x = check_value(1, "number", ...)
    local y = check_value(2, "number", ...)
    _1f:moveTo(x, y)
end

_G.old_print = _G.print
function _G.print(...)
    _1f:print(...)
end

function _G.alert(...)
    local msg = check_value(1, "string", ...)
    local title = opt_value(2, "string", "Alert", ...)
    _1f:alert(msg, title)
end

function _G.alertInfo(...)
    local msg = check_value(1, "string", ...)
    local title = opt_value(2, "string", "Information", ...)
    _1f:alertInfo(msg, title)
end

function _G.alertWarn(...)
    local msg = check_value(1, "string", ...)
    local title = opt_value(2, "string", "Warning", ...)
    _1f:alertWarn(msg, title)
end

function _G.alertError(...)
    local msg = check_value(1, "string", ...)
    local title = opt_value(2, "string", "Error", ...)
    _1f:alertError(msg, title)
end

local editorsTabPane = getEditorsTabPane()
local imagesTabPane = getImagesTabPane()

function _G.closeCurrentImageTab()
    local imgTab = imagesTabPane
    imgTab:closeTabAt(imgTab:getSelectedIndex())
end

function _G.closeCurrentEditorTab()
    local editTab = imagesTabPane
    editTab:closeTabAt(editTab:getSelectedIndex())
end

function string:ltrim()
    return (string.gsub(self, "^%s*(.-)", "%1"))
end

function string:rtrim()
    return (string.gsub(self, "(.-)%s*$", "%1"))
end

function string:trim()
    return (string.gsub(self, "^%s*(.-)%s*$", "%1"))
end

function string:random(len)
    len = len or 6
    local s = ""
    local ran = math.random
    for i=1,len do
        local n = ran(1,#self)
        s = s..self:sub(n,n)
    end
    return s
end

function string:split(sep)
    local str = self
    local strtab = {}
    while (true) do
        local pos = str:find(sep)
        if (not pos) then
            strtab[#strtab + 1] = str
            break
        end
        strtab[#strtab + 1] = str:sub(1, pos - 1)
        str = str:sub(pos + #sep, #str)
    end
    return strtab
end

require("scripts.colorpicker.init.settings")
require("scripts.colorpicker.init.customformats")
require("scripts.colorpicker.init.callbacks")
--require("scripts.tests")
